/*
 * Copyright (c) 2003-2017 Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/
/**
 * @file OSXSDComplexType.h
 * C++ run-time XML schema complex type class definition.
 */
#ifndef _OSXSDCOMPLEXTYPE_H_
#define _OSXSDCOMPLEXTYPE_H_

#include "rtxsrc/OSRTBaseType.h"
#include "rtxsrc/rtxCppDList.h"
#include "rtxmlsrc/osrtxml.h"
#include "rtxmlsrc/rtXmlCppNamespace.h"

/**
 * XSD complex type base class. This class extends the common run-time
 * base class object (OSRTBaseType).  It adds namespace attributes and
 * other meta-date structures unique to XSD complex type definitions.
 */
class EXTERNXML OSXSDComplexType : public OSRTBaseType {
 protected:
   /**
    * The mNSAttrs member variable holds a list of namespace attributes
    * that have been assigned to this type.  The list holds 'OSXMLNamespace'
    * objects.
    */
   OSRTDListClass mNSAttrs;

   /**
    * The default constructor does nothing.  The namespace attribute list
    * will initialize itself to zero entries.  It is protected to make this
    * class abstract.
    */
   OSXSDComplexType() : OSRTBaseType() {}

   /**
    * The copy constructor will make a deep-copy of the namespace list.
    */
   OSXSDComplexType (const OSXSDComplexType& o);

   /**
    * The destructor cleans up the namespace attributes list.
    */
   virtual ~OSXSDComplexType ();

   /**
    * The copy operator.
    */
   OSXSDComplexType& operator = (const OSXSDComplexType& o);

 public:
   /**
    * This method will add a namespace attribute to the namespace
    * attribute list.
    *
    * @param prefix - Namespace prefix.
    * @param uri    - Namespace URI.
    * @return Pointer to namespace record that was added or NULL if
    *   record could not be allocated.
    */
   OSXMLNamespaceClass* addNamespace
      (const OSUTF8CHAR* prefix, const OSUTF8CHAR* uri);

   /**
    * This method will add a namespace attribute to the given namespace
    * attribute list.
    *
    * @param prefix - Namespace prefix.
    * @param uri    - Namespace URI.
    * @param pNSAttrs - List of OSXSDNamespaceClass to add to
    * @return Pointer to namespace record that was added or NULL if
    *   record could not be allocated.
    */
   static OSXMLNamespaceClass* addNamespace
      (const OSUTF8CHAR* prefix, const OSUTF8CHAR* uri,
      OSRTDListClass* pNSAttrs);

   /**
    * This method will add a namespace attribute to the namespace
    * attribute list.
    *
    * @param prefix - Namespace prefix.
    * @param prefixBytes - Number of bytes to copy from the prefix string.
    * @param uri    - Namespace URI.
    * @param uriBytes - Number of bytes to copy from the uri string.
    * @return Pointer to namespace record that was added or NULL if
    *   record could not be allocated.
    */
   OSXMLNamespaceClass* addNamespace
      (const OSUTF8CHAR* prefix, size_t prefixBytes,
       const OSUTF8CHAR* uri, size_t uriBytes);

   /**
    * This method return a pointer to the namespace attribute list.
    *
    * @return Pointer to namespace attribute list.
    */
   inline OSRTDListClass* getNSAttrListPtr() { return &mNSAttrs; }


   OSXMLNamespaceClass* lookupNS
      (const OSUTF8CHAR* uriOrPrefix, OSBOOL byURI = TRUE,
       OSBOOL ignoreEmptyPrefix = FALSE);

   static OSXMLNamespaceClass* lookupNS
      (const OSUTF8CHAR* uriOrPrefix, OSRTDListClass* pNSAttrs,
      OSBOOL byURI, OSBOOL ignoreEmptyPrefix);

   /**
    * This method looks up the given URI in the namespace attributes
    * list and return the corresponding namespace record.
    *
    * @param uri    - Namespace URI.
    * @return       - Namespace record if found, or NULL if not.
    */
   inline OSXMLNamespaceClass* lookupNSURI
   (const OSUTF8CHAR* uri, OSBOOL ignoreEmptyPrefix = FALSE)
      { return lookupNS (uri, TRUE, ignoreEmptyPrefix); }

   /**
    * This method looks up the given prefix in the namespace attributes
    * list and return the corresponding namespace record.
    *
    * @param prefix    - Namespace prefix.
    * @return       - Namespace record if found, or NULL if not.
    */
   inline OSXMLNamespaceClass* lookupNSPrefix (const OSUTF8CHAR* prefix)
      { return lookupNS (prefix, FALSE, FALSE); }

#ifndef _COMPACT
   /**
    * This method prints the contents of the namespace attribute list.
    */
   void printNSAttrs();
#endif
   /**
    * This method will set a namespace attribute in the namespace
    * attribute list.  If the URI is found, the prefix for that record
    * will only be changed if the override flag is set to true.  If the
    * URI is not found, the namespace will be added.
    *
    * @param prefix - Namespace prefix.
    * @param uri    - Namespace URI.
    * @param override - Flag which indicates prefix in existing record
    *   should be changed to given prefix.
    * @return Pointer to namespace record that was added or altered or
    *   NULL if exists and no override.
    */
   OSXMLNamespaceClass* setNamespace
      (const OSUTF8CHAR* prefix, const OSUTF8CHAR* uri, OSBOOL override);

   /**
    * This function creates a QName for a qualified attribute.  If a prefix
    * is not found for the name, a new namespace entry is created with a
    * generated prefix.
    *
    * @param pctxt         Pointer to OSCTXT structure.
    * @param buf           Buffer into which qname will be written.
    * @param bufsiz        Size of the buffer.
    * @param pNS           Pointer to namespace structure.
    * @param localName     Local name of the item.
    * @return              Pointer to QName buffer (buf).
    */
   const OSUTF8CHAR* getAttrQName
      (OSCTXT* pctxt, OSUTF8CHAR* buf, size_t bufsiz,
      OSXMLNamespace* pNS, const OSUTF8CHAR* localName);

  /**
    * This function creates a QName in the given fixed-site buffer.  If
    * the name will not fit in the buffer, it is truncated.
    *
    * @param pctxt         Pointer to OSCTXT structure.
    * @param buf           Buffer into which qname will be written.
    * @param bufsiz        Size of the buffer.
    * @param pNS           Pointer to namespace structure.
    * @param localName     Local name of the item.
    * @return              Pointer to QName buffer (buf).
    */
   const OSUTF8CHAR* getQName
      (OSUTF8CHAR* buf, size_t bufsiz, OSXMLNamespace* pNS,
       const OSUTF8CHAR* localName);

  /**
    * This function returns the next unused prefix of the form "nsX" where X
    * is a number. The new namespace declaration is added to the context
    * namespace list
    *
    * @param pctxt         Pointer to OSCTXT structure
    * @param uri           Namespace URI, should not be null or empty string
    * @return              New namespace prefix.
    */
   const OSUTF8CHAR* newPrefix
      (OSCTXT* pctxt, const OSUTF8CHAR* uri);


  /**
    * This function returns the next unused prefix of the form "nsX" where X
    * is a number. The new namespace declaration is added to the context
    * namespace list
    *
    * @param pctxt         Pointer to OSCTXT structure
    * @param uri           Namespace URI, should not be null or empty string
    * @param pNSAttrs      List of OSXMLNamespaceClass to add to.
    * @return              New namespace prefix.
    */
   static const OSUTF8CHAR* newPrefix
      (OSCTXT* pctxt, const OSUTF8CHAR* uri, OSRTDListClass* pNSAttrs);
} ;

#endif
