/*
 * Copyright (c) 2003-2017 Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/
/*! \mainpage
 * <H1>C++ XML Runtime Library Classes</H1>
 *
 * The <B>C++ XML run-time classes</B> are wrapper classes that provide an
 * object-oriented interface to the common C XML run-time library functions.
 * The categories of classes provided are as follows:
 * <UL>
 * <LI>XML encode functions.</LI>
 * <LI>XML decode functions.</LI>
 * <LI>SAX interfaces.</LI>
 * <LI>Objects for global elements, namespaces, and message buffers.</LI>
 * </UL>
 */
/**
 * @file rtXmlCppXSDElement.h
 * C++ run-time XML schema global element class definition.
 */
#ifndef _RTXMLCPPXSDELEMENT_H_
#define _RTXMLCPPXSDELEMENT_H_

#include "rtxsrc/OSRTContext.h"
#include "rtxsrc/OSRTMsgBufIF.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxErrCodes.h"
#include "rtxmlsrc/osrtxml.h"

/**
 * XSD global element base class. This is the main base class for all
 * generated global element control classes. It holds a variable of a
 * generated data type as well as the associated message buffer or
 * stream class to which a message will be encoded or from which a message
 * will be decoded.
 */
class EXTERNXML OSXSDGlobalElement {
 protected:
   /**
    * The mpContext member variable holds a reference-counted C runtime
    * variable. This context is used in calls to all C run-time functions. The
    * context pointed at by this smart-pointer object is shared with the
    * message buffer object contained within this class.
    */
   OSRTCtxtPtr mpContext;

   /**
    * The mpMsgBuf member variable is a pointer to a derived message buffer or
    * stream class that will manage the message being encoded or decoded.
    */
   OSRTMessageBufferIF* mpMsgBuf;

   /**
    * The default constructor sets the message pointer member variable to NULL
    * and creates a new context object.
    */
   OSXSDGlobalElement () : mpContext (new OSRTContext()), mpMsgBuf (0) {}

   /**
    * This constructor sets the message pointer member variable to NULL and
    * initializes the context object to point at the given context value.
    *
    * @param ctxt         - Reference to a context object.
    */
   OSXSDGlobalElement (OSRTContext& ctxt) : mpContext (&ctxt), mpMsgBuf (0) {}

   /**
    * The setMsgBuf method is used to set the internal message buffer pointer
    * to point at the given message buffer or stream object.
    *
    * @param msgBuf       - Reference to a message buffer or stream object.
    */
   void setMsgBuf (OSRTMessageBufferIF& msgBuf);

 public:
   /**
    * This constructor sets the internal message buffer pointer to point at the
    * given message buffer or stream object. The context is set to point at the
    * context contained within the message buffer object. Thus, the message
    * buffer and control class object share the context. It will not be
    * released until both objects are destroyed.
    *
    * @param msgBuf       - Reference to a message buffer or stream object.
    */
   OSXSDGlobalElement (OSRTMessageBufferIF& msgBuf) : mpMsgBuf(&msgBuf) {
      mpContext = msgBuf.getContext();
   }

   /**
    * The copy constructor sets the internal message buffer pointer and context
    * to point at the message buffer and context from the original OSCType
    * object.
    *
    * @param o            - Reference to a global element object.
    */
   OSXSDGlobalElement (const OSXSDGlobalElement& o) :
      mpContext (o.mpContext), mpMsgBuf (o.mpMsgBuf) { }

   /**
    * The virtual destructor does nothing. It is overridden by derived versions
    * of this class.
    */
   virtual ~OSXSDGlobalElement () {}

   /**
    * The \c decode method decodes the message described by the
    * encapsulated message buffer object.
    */
   inline int decode () { return decodeFrom (*mpMsgBuf); }

   /**
    * The \c decodeFrom method decodes a message from the given message
    * buffer or stream argument.
    *
    * @param       - Message buffer or stream containing message to
    *                decode.
    */
   virtual int decodeFrom (OSRTMessageBufferIF&) { return 0; }

   /**
    * The \c encode method encodes a message using the encoding rules
    * specified by the derived message buffer object.
    */
   inline int encode () { return encodeTo (*mpMsgBuf); }

   /**
    * The \c encodeTo method encodes a message into the given message
    * buffer or stream argument.
    *
    * @param       - Message buffer or stream to which the message is to
    *                be encoded.
    */
   virtual int encodeTo (OSRTMessageBufferIF&) { return 0; }

   /**
    * The getCtxtPtr method returns the underlying C runtime context. This
    * context can be used in calls to C runtime functions.
    */
   inline OSCTXT* getCtxtPtr () {
      return (!mpContext.isNull ()) ? mpContext->getPtr() : 0;
   }

   /**
    * The memAlloc method allocates memory using the C runtime memory
    * management functions. The memory is tracked in the underlying context
    * structure. When both this OSXSDGlobalElement derived control class
    * object and the message buffer object are destroyed, this memory will
    * be freed.
    *
    * @param numocts      - Number of bytes of memory to allocate
    */
   inline void* memAlloc (size_t numocts) {
      return rtxMemAlloc (getCtxtPtr(), numocts);
   }

#if 0
   /**
    * The memRealloc method reallocates memory using the C runtime memory
    * management functions.
    *
    * @param ptr          - Original pointer containing dynamic memory to be
    *                       resized.
    * @param oldnumocts   - Number of bytes of memory already allocated
    * @param numocts      - Number of bytes of memory to allocate
    * @return             Reallocated memory pointer
    */
   inline void* memRealloc (void* ptr, size_t oldnumocts, size_t numocts) {
      return rtxMemRealloc (getCtxtPtr(), ptr, oldnumocts, numocts);
   }
#endif

   /**
    * The memFreePtr method frees the memory at a specific location. This
    * memory must have been allocated using the memAlloc method described
    * earlier.
    *
    * @param ptr          - Pointer to a block of memory allocated with \c
    *                       memAlloc
    */
   inline void memFreePtr (void* ptr) {
      rtxMemFreePtr (getCtxtPtr(), ptr);
   }

   /**
    * The setDefaultNamespace method sets the default namespace for
    * the element to the given value.
    *
    * @param uri - Default namespace URI
    */
   inline void setDefaultNamespace (const OSUTF8CHAR* uri) {
      setNamespace (0, uri);
   }

   /**
    * The setDiag method turns diagnostic tracing on or off.
    *
    * @param value - Boolean on/off value (default = on)
    */
   inline void setDiag (OSBOOL value = TRUE) {
      rtxSetDiag (getCtxtPtr(), value);
   }

   /**
    * The setEncXSINamespace method sets a flag in the internal context
    * that indicates the xsi namespace attribute must be encoded.
    *
    * @param value - Boolean on/off value (default = on)
    */
   inline void setEncXSINamespace (OSBOOL value = TRUE) {
      rtXmlSetEncXSINamespace (getCtxtPtr(), value);
   }

   /**
    * The setNamespace method adds or modifies the namespace with the
    * given URI in the namespace list to contain the given prefix.
    *
    * @param prefix - Namespace prefix
    * @param uri - Namespace URI
    */
   inline void setNamespace (const OSUTF8CHAR* prefix, const OSUTF8CHAR* uri) {
      mpMsgBuf->setNamespace (prefix, uri);
   }

   /**
    * The setNoNSSchemaLocation method adds an xsi:noNamespaceSchemaLocation
    * attribute to the document.
    *
    * @param uri - URI for noNamespaceSchemaLocation.
    */
   inline void setNoNSSchemaLocation (const OSUTF8CHAR* uri) {
      rtXmlSetNoNSSchemaLocation (mpContext->getPtr(), uri);
   }

   /**
    * The setSchemaLocation method adds an xsi:schemaLocation
    * attribute to the document.
    *
    * @param uri - URI for schemaLocation.
    */
   inline void setSchemaLocation (const OSUTF8CHAR* uri) {
      rtXmlSetSchemaLocation (mpContext->getPtr(), uri);
   }

   /**
    * The setXSIType method sets a type name to be used in the
    * xsi:type attribute in the top-level module element declaration.
    *
    * @param typeName - XSI type name
    */
   inline void setXSIType (const OSUTF8CHAR* typeName) {
      rtXmlSetXSITypeAttr (mpContext->getPtr(), typeName);
   }

   /**
    * The \c validate method validates the message described by the
    * encapsulated message buffer object.
    */
   inline int validate () { return validateFrom (*mpMsgBuf); }

   /**
    * The \c validateFrom method validates a message from the given message
    * buffer or stream argument.
    *
    * @param       - Message buffer or stream containing message to
    *                 validate.
    */
   virtual int validateFrom (OSRTMessageBufferIF&)
      { return RTERR_NOTSUPP; }
} ;

#endif
